==> kite-0.1.tar.gz: kite-0.1/KiteDynamicObject.cpp <==
#include "KiteVM.h"
#include "KiteObject.h"
#include "KiteCode.h"
#include "KiteString.h"
#include "KiteTree.h"
#include "KiteIdentifier.h"
#include "KiteList.h"
#include "KiteDynamicObject.h"

using namespace KiteVM;

KiteDynamicObject::KiteDynamicObject(KiteVM::VM &vm, KiteReference<KiteObject> parent, KiteVM::Instruction *ip)
: KiteObject(true), instance(false), inherits(parent) {
    classInfoInit(vm, ip);
}

void KiteDynamicObject::classInfoInit(KiteVM::VM &vm, KiteVM::Instruction *ip) {
    // Execute code to create static values
	vm.syms.addContext();
	KiteReference<KiteObject> test(vm.run(ip));
	if (typeid(*test) == typeid(KiteTree)) {
		// Error while running construction code
		throw test;
	}

	// Save symbols created
	syms = vm.syms.getTop();
	vm.syms.removeContext();
}

KiteDynamicObject::KiteDynamicObject(KiteVM::VM &vm, KiteVM::Instruction *ip) : KiteObject(true), instance(false), inherits(KiteObject::getClassinfo()) {
	classInfoInit(vm, ip);
}

KiteDynamicObject::KiteDynamicObject(KiteVM::VM &vm, KiteReference<KiteObject> base, KiteReference<KiteObject> param) 
: KiteObject(true), instance(true), classinfo(base) { }

KiteReference<KiteObject>
KiteDynamicObject::create(KiteVM::VM &vm, KiteReference<KiteObject> base, KiteReference<KiteObject> param) {
	KiteReference<KiteObject> ret(new KiteDynamicObject(vm, base, param));

	// Call constructor
	vm.syms.addContext();
	vm.syms.addVar(KiteReference<KiteIdentifier>(new KiteIdentifier("this")), ret);
	KiteIdentifier *id = new KiteIdentifier("new");
	KiteReference<KiteObject> fun = (*ret).dereference(vm, KiteReference<KiteIdentifier>(id));
	(void)(*fun).call(vm, param); // Call constructor
	vm.syms.removeContext();

	return ret;
}

KiteDynamicObject::~KiteDynamicObject() {
	// Everything will be deconstructed by default
}

KiteReference<KiteObject> KiteDynamicObject::dereference(VM &vm, KiteReference<KiteIdentifier> id) {
	// Go through class hierachy, looking for object
	KiteObject *cur = this;
	bool staticobjs = (isInstance() ? false : true);

	while(cur) {
		if (typeid(*cur) == typeid(KiteDynamicObject)) {
			if (dynamic_cast<KiteDynamicObject*>(cur)->syms.count(*id) > 0) {
				return dynamic_cast<KiteDynamicObject*>(cur)->syms[*id];
			}
		} else {
			if (cur->getSyms().count(id) > 0) {
				return cur->getSyms()[id];
			}
		}

		if (!staticobjs) {
			staticobjs = true;
			cur = &*(dynamic_cast<KiteDynamicObject&>(*cur).classinfo);
		} else {
			cur = &(*cur->getInherits());
		}
	}

	// We can't find the object, so time to create it.
	return syms[*id];
}

==> kite-0.2.tar.gz: kite-0.2/KiteDynamicObject.cpp <==
#include "KiteVM.h"
#include "KiteObject.h"
#include "KiteCode.h"
#include "KiteString.h"
#include "KiteTree.h"
#include "KiteIdentifier.h"
#include "KiteList.h"
#include "KiteDynamicObject.h"

using namespace KiteVM;

KiteDynamicObject::KiteDynamicObject(KiteVM::VM &vm, KiteReference<KiteObject> parent, KiteVM::Instruction *ip)
: KiteObject(true), instance(false), inherits(parent) {
    classInfoInit(vm, ip);
}

void KiteDynamicObject::classInfoInit(KiteVM::VM &vm, KiteVM::Instruction *ip) {
    // Execute code to create static values
	vm.syms.addContext();
	KiteReference<KiteObject> test(vm.run(ip));
	if (typeid(*test) == typeid(KiteTree)) {
		// Error while running construction code
		throw test;
	}

	// Save symbols created
	syms = vm.syms.getTop();
	vm.syms.removeContext();
}

KiteDynamicObject::KiteDynamicObject(KiteVM::VM &vm, KiteVM::Instruction *ip) : KiteObject(true), instance(false), inherits(KiteObject::getClassinfo()) {
	classInfoInit(vm, ip);
}

KiteDynamicObject::KiteDynamicObject(KiteVM::VM &vm, KiteReference<KiteObject> base, KiteReference<KiteObject> param) 
: KiteObject(true), instance(true), classinfo(base) { }

KiteReference<KiteObject>
KiteDynamicObject::create(KiteVM::VM &vm, KiteReference<KiteObject> base, KiteReference<KiteObject> param) {
	KiteReference<KiteObject> ret(new KiteDynamicObject(vm, base, param));

	// Call constructor
	vm.syms.addContext();
	vm.syms.addVar("this", ret);
	KiteReference<KiteObject> fun = (*ret).dereference(vm, "new");
	(void)(*fun).call(vm, param); // Call constructor
	vm.syms.removeContext();

	return ret;
}

KiteDynamicObject::~KiteDynamicObject() {
	// Everything will be deconstructed by default
}

KiteReference<KiteObject> KiteDynamicObject::dereference(VM &vm, string id) {
	// Go through class hierachy, looking for object
	KiteObject *cur = this;
	bool staticobjs = (isInstance() ? false : true);

	while(cur) {
		if (typeid(*cur) == typeid(KiteDynamicObject)) {
			if (((KiteDynamicObject*)(cur))->syms.count(id) > 0) {
				return ((KiteDynamicObject*)(cur))->syms[id];
			}
		} else {
			if (cur->getSyms().count(id) > 0) {
				return cur->getSyms()[id];
			}
		}

		if (!staticobjs) {
			staticobjs = true;
			cur = &*(((KiteDynamicObject&)(*cur)).classinfo);
		} else {
			cur = &(*cur->getInherits());
		}
	}

	// We can't find the object, so time to create it.
	return syms[id];
}
